package jtelnet;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.ConnectException;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;

public class JTelnet {

	private String ip;
	private int port;
	private List<DisplayDriver> displays;
	private Timer scheduler;
	private SocketReader sockReader;
	private SocketWriter sockWriter;
	private Socket sock;
	
	public JTelnet(String ip, int port) {
		this.ip = ip;
		this.port = port;
		this.displays = new LinkedList<DisplayDriver>();
		this.scheduler = new Timer();
	}

	public JTelnet() {
		this.displays = new LinkedList<DisplayDriver>();
		this.scheduler = new Timer();
	}
	
	public String getIp() {
		return ip;
	}

	public void setIp(String ip) {
		this.ip = ip;
	}

	public int getPort() {
		return port;
	}

	public void setPort(int port) {
		this.port = port;
	}

	public static void main(String[] args) throws UnknownHostException, IOException {
		if (args.length != 2 && args.length != 1 ) {
			System.out.println("Usage: JTelnet IP [PORT]");
		} else {
			int port = 23;
			if (args.length == 2)
				port = Integer.parseInt(args[1]);
			JTelnet jtelnet = new JTelnet(args[0],port);
			DisplayDriver display = new MonitorDisplayDriver();
			jtelnet.addDisplayDriver(display);
			jtelnet.connect();
			BufferedReader lettore = new BufferedReader(new InputStreamReader(System.in));
			while (true) {
				jtelnet.sendCommand(lettore.readLine());
			}
		}
	}

	public void connect() throws UnknownHostException, IOException {
		try {
			InetAddress ipAddress = InetAddress.getByName(this.ip);
			this.notifyDisplays("-- Connecting to " + ipAddress.getHostName() + " (" + ipAddress.getHostAddress() + ") ...");
			InetSocketAddress dest = new InetSocketAddress(ipAddress,this.port);
			Socket sock = new Socket();
			sock.connect(dest,5000);
			this.notifyDisplays("-- Connection done.");
			this.sock = sock;
			this.sockReader = new SocketReader(sock.getInputStream(),this);
			this.sockWriter = new SocketWriter(sock.getOutputStream());
			this.scheduler.scheduleAtFixedRate((TimerTask) this.sockReader,100,200);
		} catch (ConnectException r) {
			this.notifyDisplays("-- Cannot make connection. Aborting...");
			System.exit(0);
		} catch (UnknownHostException h) {
			this.notifyDisplays("-- Cannot find host. Aborting...");
			System.exit(0);
		} catch (IOException e) {
			this.notifyDisplays("-- Cannot connect to port " + this.port + ". Aborting...");
			System.exit(0);
		} 
		
	}

	public void addDisplayDriver(DisplayDriver display) {
		this.displays.add(display);
	}
	
	public void notifyDisplays(String notify) {
		for(Iterator<DisplayDriver> it = this.displays.iterator();it.hasNext();)
			it.next().notify(notify);
	}
	
	public boolean sendCommand(String text) {
		return this.sockWriter.sendCommand(text);
	}
	
	public void closeConnection() {
		try {
			this.sock.close();
			this.notifyDisplays("-- Connection closed.");
		} catch (IOException e) {
			this.notifyDisplays("-- Connection closed.");
		}
	}
	
	public void exitOnClose() {
		closeConnection();
		this.notifyDisplays("-- Exiting...");
		System.exit(0);
	}
	
	public boolean isNotConnected() {
		return !this.sock.isConnected();
	}
	
}
